#!/usr/bin/env python

# Copyright 2015 The Kubernetes Authors All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import sys
# This script helps parse out the private IP addreses from the
# `juju run` command's JSON object, see cluster/juju/util.sh

if len(sys.argv) > 1:
    # It takes the JSON output as the first argument.
    nodes = json.loads(sys.argv[1])
    # There can be multiple nodes to print the Stdout.
    for num in nodes:
        print num['Stdout'].rstrip()
else:
    exit(1)



#!/usr/bin/env python

# Copyright 2015 The Kubernetes Authors All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import shlex
import subprocess
from path import path


def run(command, shell=False):
    """ A convience method for executing all the commands. """
    print(command)
    if shell is False:
        command = shlex.split(command)
    output = subprocess.check_output(command, shell=shell)
    print(output)
    return output


class KubernetesInstaller():
    """
    This class contains the logic needed to install kuberentes binary files.
    """

    def __init__(self, arch, version, output_dir):
        """ Gather the required variables for the install. """
        # The kubernetes-master charm needs certain commands to be aliased.
        self.aliases = {'kube-apiserver': 'apiserver',
                        'kube-controller-manager': 'controller-manager',
                        'kube-proxy': 'kube-proxy',
                        'kube-scheduler': 'scheduler',
                        'kubectl': 'kubectl',
                        'kubelet': 'kubelet'}
        self.arch = arch
        self.version = version
        self.output_dir = path(output_dir)

    def build(self, branch):
        """ Build kubernetes from a github repository using the Makefile. """
        # Remove any old build artifacts.
        make_clean = 'make clean'
        run(make_clean)
        # Always checkout the master to get the latest repository information.
        git_checkout_cmd = 'git checkout master'
        run(git_checkout_cmd)
        # When checking out a tag, delete the old branch (not master).
        if branch != 'master':
            git_drop_branch = 'git branch -D {0}'.format(self.version)
            print(git_drop_branch)
            rc = subprocess.call(git_drop_branch.split())
            if rc != 0:
                print('returned: %d' % rc)
        # Make sure the git repository is up-to-date.
        git_fetch = 'git fetch origin {0}'.format(branch)
        run(git_fetch)

        if branch == 'master':
            git_reset = 'git reset --hard origin/master'
            run(git_reset)
        else:
            # Checkout a branch of kubernetes so the repo is correct.
            checkout = 'git checkout -b {0} {1}'.format(self.version, branch)
            run(checkout)

        # Create an environment with the path to the GO binaries included.
        go_path = ('/usr/local/go/bin', os.environ.get('PATH', ''))
        go_env = os.environ.copy()
        go_env['PATH'] = ':'.join(go_path)
        print(go_env['PATH'])

        # Compile the binaries with the make command using the WHAT variable.
        make_what = "make all WHAT='cmd/kube-apiserver cmd/kubectl "\
                    "cmd/kube-controller-manager plugin/cmd/kube-scheduler "\
                    "cmd/kubelet cmd/kube-proxy'"
        print(make_what)
        rc = subprocess.call(shlex.split(make_what), env=go_env)

    def install(self, install_dir=path('/usr/local/bin')):
        """ Install kubernetes binary files from the output directory. """

        if not install_dir.isdir():
            install_dir.makedirs_p()

        # Create the symbolic links to the real kubernetes binaries.
        for key, value in self.aliases.iteritems():
            target = self.output_dir / key
            if target.exists():
                link = install_dir / value
                if link.exists():
                    link.remove()
                target.symlink(link)
            else:
                print('Error target file {0} does not exist.'.format(target))
                exit(1)



#!/usr/bin/env python

# Copyright 2015 The Kubernetes Authors All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import httplib
import json
import time


class Registrator:

    def __init__(self):
        self.ds ={
          "creationTimestamp": "",
          "kind": "Minion",
          "name": "", # private_address
          "metadata": {
            "name": "", #private_address,
          },
          "spec": {
            "externalID": "", #private_address
            "capacity": {
                "mem": "",  # mem + ' K',
                "cpu": "",  # cpus
            }
          },
          "status": {
            "conditions": [],
            "hostIP": "", #private_address
          }
        }

    @property
    def data(self):
        ''' Returns a data-structure for population to make a request. '''
        return self.ds

    def register(self, hostname, port, api_path):
        ''' Contact the API Server for a new registration '''
        headers = {"Content-type": "application/json",
                   "Accept": "application/json"}
        connection = httplib.HTTPConnection(hostname, port)
        print 'CONN {}'.format(connection)
        connection.request("POST", api_path, json.dumps(self.data), headers)
        response = connection.getresponse()
        body = response.read()
        print(body)
        result = json.loads(body)
        print("Response status:%s reason:%s body:%s" % \
             (response.status, response.reason, result))
        return response, result

    def update(self):
        ''' Contact the API Server to update a registration '''
        # do a get on the API for the node
        # repost to the API with any modified data
        pass

    def save(self):
        ''' Marshall the registration data '''
        # TODO
        pass

    def command_succeeded(self, response, result):
        ''' Evaluate response data to determine if the command was successful '''
        if response.status in [200, 201]:
            print("Registered")
            return True
        elif response.status in [409,]:
            print("Status Conflict")
            # Suggested return a PUT instead of a POST with this response
            # code, this predicates use of the UPDATE method
            # TODO
        elif response.status in (500,) and result.get(
            'message', '').startswith('The requested resource does not exist'):
            # There's something fishy in the kube api here (0.4 dev), first time we
            # go to register a new minion, we always seem to get this error.
            # https://github.com/GoogleCloudPlatform/kubernetes/issues/1995
            time.sleep(1)
            print("Retrying registration...")
            raise ValueError("Registration returned 500, retry")
            # return register_machine(apiserver, retry=True)
        else:
            print("Registration error")
            # TODO - get request data
            raise RuntimeError("Unable to register machine with")
